@@ -37,6 +37,9 @@
  *
  * Revisions   :
  *    $Log: acconfig.h,v $
+ *    Revision 1.33  2006/12/17 19:15:26  fabiankeil
+ *    Added ./configure switch for FEATURE_GRACEFUL_TERMINATION.
+ *
  *    Revision 1.32  2006/07/18 14:48:45  david__schmidt
  *    Reorganizing the repository: swapping out what was HEAD (the old 3.1 branch)
  *    with what was really the latest development (the v_3_0_branch branch)
@@ -344,6 +347,11 @@
  */
 #undef FEATURE_NO_GIFS
 
+/*
+ * Allow to shutdown Privoxy through the webinterface.
+ */
+#undef FEATURE_GRACEFUL_TERMINATION
+
 /*
  * Use POSIX threads instead of native threads.
  */
