@@ -1,4 +1,4 @@
-const char encode_rcs[] = "$Id: encode.c,v 1.10 2006/07/18 14:48:45 david__schmidt Exp $";
+const char encode_rcs[] = "$Id: encode.c,v 1.13 2007/08/18 14:34:27 fabiankeil Exp $";
 /*********************************************************************
  *
  * File        :  $Source: /cvsroot/ijbswa/current/encode.c,v $
@@ -33,6 +33,15 @@ const char encode_rcs[] = "$Id: encode.c,v 1.10 2006/07/18 14:48:45 david__schmi
  *
  * Revisions   :
  *    $Log: encode.c,v $
+ *    Revision 1.13  2007/08/18 14:34:27  fabiankeil
+ *    Make xtoi() extern so it can be used in pcrs.c.
+ *
+ *    Revision 1.12  2007/08/04 10:15:51  fabiankeil
+ *    Use strlcpy() instead of strcpy().
+ *
+ *    Revision 1.11  2006/12/28 18:25:53  fabiankeil
+ *    Fixed gcc43 compiler warning.
+ *
  *    Revision 1.10  2006/07/18 14:48:45  david__schmidt
  *    Reorganizing the repository: swapping out what was HEAD (the old 3.1 branch)
  *    with what was really the latest development (the v_3_0_branch branch)
@@ -73,7 +82,9 @@ const char encode_rcs[] = "$Id: encode.c,v 1.10 2006/07/18 14:48:45 david__schmi
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 
+#include "miscutil.h"
 #include "encode.h"
 
 const char encode_h_rcs[] = ENCODE_H_VERSION;
@@ -189,6 +200,7 @@ static const char * const cookie_code_map[256] = {
 char * html_encode(const char *s)
 {
    char * buf;
+   size_t buf_size;
    
    if (s == NULL)
    {
@@ -196,7 +208,8 @@ char * html_encode(const char *s)
    }
 
    /* each input char can expand to at most 6 chars */
-   buf = (char *) malloc((strlen(s) * 6) + 1);
+   buf_size = (strlen(s) * 6) + 1;
+   buf = (char *) malloc(buf_size);
 
    if (buf)
    {
@@ -207,8 +220,9 @@ char * html_encode(const char *s)
          const char * replace_with = html_code_map[(unsigned char) c];
          if(replace_with != NULL)
          {
-            strcpy(p, replace_with);
-            p += strlen(replace_with);
+            const size_t bytes_written = (size_t)(p - buf);
+            assert(bytes_written < buf_size);
+            p += strlcpy(p, replace_with, buf_size - bytes_written);
          }
          else
          {
@@ -219,6 +233,7 @@ char * html_encode(const char *s)
       *p = '\0';
    }
 
+   assert(strlen(buf) < buf_size);
    return(buf);
 }
 
@@ -276,6 +291,7 @@ char * html_encode_and_free_original(char *s)
 char * cookie_encode(const char *s)
 {
    char * buf;
+   size_t buf_size;
 
    if (s == NULL)
    {
@@ -283,7 +299,8 @@ char * cookie_encode(const char *s)
    }
 
    /* each input char can expand to at most 3 chars */
-   buf = (char *) malloc((strlen(s) * 3) + 1);
+   buf_size = (strlen(s) * 3) + 1;
+   buf = (char *) malloc(buf_size);
 
    if (buf)
    {
@@ -294,8 +311,9 @@ char * cookie_encode(const char *s)
          const char * replace_with = cookie_code_map[(unsigned char) c];
          if (replace_with != NULL)
          {
-            strcpy(p, replace_with);
-            p += strlen(replace_with);
+            const size_t bytes_written = (size_t)(p - buf);
+            assert(bytes_written < buf_size);
+            p += strlcpy(p, replace_with, buf_size - bytes_written);
          }
          else
          {
@@ -306,6 +324,7 @@ char * cookie_encode(const char *s)
       *p = '\0';
    }
 
+   assert(strlen(buf) < buf_size);
    return(buf);
 }
 
@@ -328,6 +347,7 @@ char * cookie_encode(const char *s)
 char * url_encode(const char *s)
 {
    char * buf;
+   size_t buf_size;
 
    if (s == NULL)
    {
@@ -335,7 +355,8 @@ char * url_encode(const char *s)
    }
 
    /* each input char can expand to at most 3 chars */
-   buf = (char *) malloc((strlen(s) * 3) + 1);
+   buf_size = (strlen(s) * 3) + 1;
+   buf = (char *) malloc(buf_size);
 
    if (buf)
    {
@@ -346,8 +367,9 @@ char * url_encode(const char *s)
          const char * replace_with = url_code_map[(unsigned char) c];
          if (replace_with != NULL)
          {
-            strcpy(p, replace_with);
-            p += strlen(replace_with);
+            const size_t bytes_written = (size_t)(p - buf);
+            assert(bytes_written < buf_size);
+            p += strlcpy(p, replace_with, buf_size - bytes_written);
          }
          else
          {
@@ -359,6 +381,7 @@ char * url_encode(const char *s)
 
    }
 
+   assert(strlen(buf) < buf_size);
    return(buf);
 }
 
@@ -409,7 +432,7 @@ static int xdtoi(const int d)
  * Returns     :  The integer value, or 0 for non-hex strings.
  *
  *********************************************************************/
-static int xtoi(const char *s)
+int xtoi(const char *s)
 {
    int d1, d2;
 
@@ -458,7 +481,7 @@ char *url_decode(const char * s)
                break;
 
             case '%':
-               if ((*q = xtoi(s + 1)) != '\0')
+               if ((*q = (char)xtoi(s + 1)) != '\0')
                {
                   s += 3;
                   q++;
